@@ -3,7 +3,6 @@
 #   Codec encoding tests for HongKong encodings.
 #
 
-from test import support
 from test import multibytecodec_support
 import unittest
 
@@ -19,8 +18,5 @@ class Test_Big5HKSCS(multibytecodec_support.TestBase, unittest.TestCase):
         (b"abc\x80\x80\xc1\xc4", "ignore",  "abc\u8b10"),
     )
 
-def test_main():
-    support.run_unittest(__name__)
-
 if __name__ == "__main__":
-    test_main()
+    unittest.main()
